searchNodes=[{"type":"module","title":"EctoSparkles","doc":"","ref":"EctoSparkles.html"},{"type":"macro","title":"EctoSparkles.join_preload/2","doc":"join_preload is a helper for preloading associations using joins. By default, Ecto preloads associations using a separate query for each association, which can degrade performance. You could make it run faster by using a combination of join/preload, but that requires a bit of boilerplate (see example below). With EctoSparkles , you can accomplish this with just one line of code. Example using just Ecto import Ecto.Query Invoice |&gt; join ( :left , [ i ] , assoc ( i , :customer ) , as : :customer ) |&gt; join ( :left , [ i , c ] , assoc ( c , :account ) , as : :account ) |&gt; join ( :left , [ i ] , assoc ( i , :lines ) , as : :lines ) |&gt; preload ( [ lines : v , customers : c , account : a ] , lines : v , customer : { c , [ a : account ] } ) |&gt; Repo . all ( ) Example using join_preload import EctoSparkles Invoice |&gt; join_preload ( [ :customer , :account ] ) |&gt; join_preload ( [ :lines ] ) |&gt; Repo . all ( )","ref":"EctoSparkles.html#join_preload/2"},{"type":"macro","title":"EctoSparkles.proload/3","doc":"AKA join_preload++ . It's more powerful, but it does it with more (and different!) syntax. e.g. proload ( query , activity : [ :verb , :boost_count , :like_count , :replied , # relations under object will have their aliases prefixed with object_, i.e. # :object_message, :object_post, :object_post_content # the original names will still be used for the associations. object : { &quot;object_&quot; , [ :message , :post , :post_content ] } ] )","ref":"EctoSparkles.html#proload/3"},{"type":"macro","title":"EctoSparkles.reusable_join/5","doc":"reusable_join is similar to Ecto.Query.join/{4,5} , but can be called multiple times with the same alias. Note that only the first join operation is performed, the subsequent ones that use the same alias are just ignored. Also note that because of this behaviour, it is mandatory to specify an alias when using this function. This is helpful when you need to perform a join while building queries one filter at a time, because the same filter could be used multiple times or you could have multiple filters that require the same join, which poses a problem with how the filter/3 callback work, as you need to return a dynamic with the filtering, which means that the join must have an alias, and by default Ecto raises an error when you add multiple joins with the same alias. To solve this, it is recommended to use this macro instead of the default Ecto.Query.join/{4,5} , in which case there will be only one join in the query that can be reused by multiple filters.","ref":"EctoSparkles.html#reusable_join/5"},{"type":"module","title":"EctoSparkles.AutoMigrator","doc":"Runs ecto migrations automatically on startup (add this to your app's supervision tree)","ref":"EctoSparkles.AutoMigrator.html"},{"type":"function","title":"EctoSparkles.AutoMigrator.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"EctoSparkles.AutoMigrator.html#child_spec/1"},{"type":"function","title":"EctoSparkles.AutoMigrator.init/1","doc":"Callback implementation for GenServer.init/1 .","ref":"EctoSparkles.AutoMigrator.html#init/1"},{"type":"function","title":"EctoSparkles.AutoMigrator.start_link/1","doc":"","ref":"EctoSparkles.AutoMigrator.html#start_link/1"},{"type":"function","title":"EctoSparkles.AutoMigrator.startup_migrations/0","doc":"","ref":"EctoSparkles.AutoMigrator.html#startup_migrations/0"},{"type":"module","title":"EctoSparkles.Changesets.Errors","doc":"","ref":"EctoSparkles.Changesets.Errors.html"},{"type":"function","title":"EctoSparkles.Changesets.Errors.changeset_errors_string/2","doc":"","ref":"EctoSparkles.Changesets.Errors.html#changeset_errors_string/2"},{"type":"function","title":"EctoSparkles.Changesets.Errors.error/2","doc":"","ref":"EctoSparkles.Changesets.Errors.html#error/2"},{"type":"behaviour","title":"EctoSparkles.DataMigration","doc":"A behaviour implemented by our data migrations (generally backfills). Based on A microframework for backfill migrations in Elixir's Ecto , in turn based on David Bernheisel's template for deterministic backfills . A data migration using this behaviour may look like this (which you can put simply put in Ecto migrations, eg. priv/repo/migrations/priv/repo/migrations/20231019004944_data_onboarding_step.exs ): defmodule MyApp.Repo.Migrations.BackfillOnboardingStep do alias EctoSparkles.DataMigration use DataMigration @impl DataMigration def base_query do # NOTE: This works in cases where: # 1. The data can be queried with a condition that not longer applies after the migration ran, so you can repeatedly query the data and update the data until the query result is empty. For example, if a column is currently null and will be updated to not be null, then you can query for the null records and pick up where you left off. # 2. The migration is written in such a way that it can be ran several times on the same data without causing data loss or duplication (or crashing). from ( u in &quot;users&quot; , # Notice how we do not use Ecto schemas here. where : is_nil ( u . onboarding_step ) , select : %{ id : u . id } ) end @impl DataMigration def config do % DataMigration.Config { batch_size : 100 , throttle_ms : 1_000 , repo : MyApp.Repo } end @impl DataMigration def migrate ( results ) do Enum . each ( results , fn %{ id : user_id } -&gt; # hooks into a context module, which is more likely to be kept up to date as the app evolves, to avoid having to update old migrations user_id |&gt; MyApp.Users . set_onboarding_step! ( ) end ) end end","ref":"EctoSparkles.DataMigration.html"},{"type":"callback","title":"EctoSparkles.DataMigration.base_query/0","doc":"The core of the query you want to use to SELECT a map of your data. The DataMigration.Runner will take care of limiting this to a batch size, ordering it by row ID, and restricting it to rows you haven't yet handled. The query must select a map, and that map must have an :id key for the migration runner to reference as the last-modified row in your table.","ref":"EctoSparkles.DataMigration.html#c:base_query/0"},{"type":"callback","title":"EctoSparkles.DataMigration.config/0","doc":"","ref":"EctoSparkles.DataMigration.html#c:config/0"},{"type":"callback","title":"EctoSparkles.DataMigration.migrate/1","doc":"The callback to operate on a result set from your query. Implementers should raise an error if you're unable to process the batch.","ref":"EctoSparkles.DataMigration.html#c:migrate/1"},{"type":"module","title":"EctoSparkles.DataMigration.Config","doc":"Configuration for a DataMigration behaviour module, used by the DataMigration.Runner . batch size: how many elements from your table to migrate at a time. throttle time: the amount of downtime the runner should sleep between batches. async: Whether to run the migration in an async process, meaning the execution of the rest of the migrations (and the app startup if you're auto-migrating on start) won't be delayed. WARNING: this means the migration will be marked as done as soon as it starts, so if the process is interrupted it won't be re-run automatically. first_id: The very first ID when sorting UUIDs in ascending order. If you use integer IDs instead, this would be 0.","ref":"EctoSparkles.DataMigration.Config.html"},{"type":"module","title":"EctoSparkles.DataMigration.Runner","doc":"Runs a DataMigration","ref":"EctoSparkles.DataMigration.Runner.html"},{"type":"function","title":"EctoSparkles.DataMigration.Runner.run/1","doc":"","ref":"EctoSparkles.DataMigration.Runner.html#run/1"},{"type":"module","title":"EctoSparkles.ErlangTermBinary","doc":"A custom Ecto type for handling the serialization of arbitrary data types stored as binary data in the database. Requires the underlying DB field to be a binary.","ref":"EctoSparkles.ErlangTermBinary.html"},{"type":"function","title":"EctoSparkles.ErlangTermBinary.cast/1","doc":"Callback implementation for Ecto.Type.cast/1 .","ref":"EctoSparkles.ErlangTermBinary.html#cast/1"},{"type":"function","title":"EctoSparkles.ErlangTermBinary.cast/2","doc":"Provides custom casting rules for params. Nothing changes here. We only need to handle deserialization.","ref":"EctoSparkles.ErlangTermBinary.html#cast/2"},{"type":"function","title":"EctoSparkles.ErlangTermBinary.dump/1","doc":"Converting the data structure to binary for storage.","ref":"EctoSparkles.ErlangTermBinary.html#dump/1"},{"type":"function","title":"EctoSparkles.ErlangTermBinary.embed_as/1","doc":"Callback implementation for Ecto.Type.embed_as/1 .","ref":"EctoSparkles.ErlangTermBinary.html#embed_as/1"},{"type":"function","title":"EctoSparkles.ErlangTermBinary.equal?/2","doc":"Callback implementation for Ecto.Type.equal?/2 .","ref":"EctoSparkles.ErlangTermBinary.html#equal?/2"},{"type":"function","title":"EctoSparkles.ErlangTermBinary.load/1","doc":"Convert the raw binary value from the database back to the desired term. Uses Plug.Crypto.non_executable_binary_to_term/2 - a restricted version of :erlang.binary_to_term/2 that forbids executable terms, such as anonymous functions. By default this function does not restrict atoms, except if compiled in prod env, then the [:safe] option is set, so only existing (and loaded) atoms will be deserialized.","ref":"EctoSparkles.ErlangTermBinary.html#load/1"},{"type":"function","title":"EctoSparkles.ErlangTermBinary.type/0","doc":"Callback implementation for Ecto.Type.type/0 .","ref":"EctoSparkles.ErlangTermBinary.html#type/0"},{"type":"module","title":"EctoSparkles.Log","doc":"Log slow Ecto queries","ref":"EctoSparkles.Log.html"},{"type":"function","title":"EctoSparkles.Log.check_if_n_plus_1/1","doc":"","ref":"EctoSparkles.Log.html#check_if_n_plus_1/1"},{"type":"function","title":"EctoSparkles.Log.format_log/4","doc":"","ref":"EctoSparkles.Log.html#format_log/4"},{"type":"function","title":"EctoSparkles.Log.format_stacktrace_sliced/3","doc":"","ref":"EctoSparkles.Log.html#format_stacktrace_sliced/3"},{"type":"function","title":"EctoSparkles.Log.handle_event/4","doc":"","ref":"EctoSparkles.Log.html#handle_event/4"},{"type":"function","title":"EctoSparkles.Log.inline_params/3","doc":"","ref":"EctoSparkles.Log.html#inline_params/3"},{"type":"function","title":"EctoSparkles.Log.log_query/4","doc":"","ref":"EctoSparkles.Log.html#log_query/4"},{"type":"function","title":"EctoSparkles.Log.setup/1","doc":"","ref":"EctoSparkles.Log.html#setup/1"},{"type":"module","title":"EctoSparkles.Migrator","doc":"","ref":"EctoSparkles.Migrator.html"},{"type":"function","title":"EctoSparkles.Migrator.create/0","doc":"","ref":"EctoSparkles.Migrator.html#create/0"},{"type":"function","title":"EctoSparkles.Migrator.create/2","doc":"","ref":"EctoSparkles.Migrator.html#create/2"},{"type":"function","title":"EctoSparkles.Migrator.migrate/0","doc":"","ref":"EctoSparkles.Migrator.html#migrate/0"},{"type":"function","title":"EctoSparkles.Migrator.migrate/1","doc":"","ref":"EctoSparkles.Migrator.html#migrate/1"},{"type":"function","title":"EctoSparkles.Migrator.rollback/2","doc":"","ref":"EctoSparkles.Migrator.html#rollback/2"},{"type":"function","title":"EctoSparkles.Migrator.rollback_all/0","doc":"","ref":"EctoSparkles.Migrator.html#rollback_all/0"},{"type":"function","title":"EctoSparkles.Migrator.rollback_all/1","doc":"","ref":"EctoSparkles.Migrator.html#rollback_all/1"},{"type":"function","title":"EctoSparkles.Migrator.rollback_to/1","doc":"","ref":"EctoSparkles.Migrator.html#rollback_to/1"},{"type":"function","title":"EctoSparkles.Migrator.rollback_to/2","doc":"","ref":"EctoSparkles.Migrator.html#rollback_to/2"},{"type":"function","title":"EctoSparkles.Migrator.status/0","doc":"Print the migration status for configured Repos' migrations.","ref":"EctoSparkles.Migrator.html#status/0"},{"type":"module","title":"EctoSparkles.NPlus1Detector","doc":"Checks a query against the previous one and increments counter of collisions or swaps previous query with the last one.","ref":"EctoSparkles.NPlus1Detector.html"},{"type":"function","title":"EctoSparkles.NPlus1Detector.check/1","doc":"","ref":"EctoSparkles.NPlus1Detector.html#check/1"},{"type":"function","title":"EctoSparkles.NPlus1Detector.do_check/2","doc":"","ref":"EctoSparkles.NPlus1Detector.html#do_check/2"},{"type":"module","title":"EctoSparkles.SanitiseStrings","doc":"Provides functions for sanitising input on Ecto.Changeset string fields.","ref":"EctoSparkles.SanitiseStrings.html"},{"type":"function","title":"EctoSparkles.SanitiseStrings.clean_html/2","doc":"","ref":"EctoSparkles.SanitiseStrings.html#clean_html/2"},{"type":"function","title":"EctoSparkles.SanitiseStrings.sanitise_strings/2","doc":"","ref":"EctoSparkles.SanitiseStrings.html#sanitise_strings/2"},{"type":"function","title":"EctoSparkles.SanitiseStrings.strip_all_tags/2","doc":"Sanitises all changes in the given changeset that apply to field which are of the :string Ecto type. By default it uses the HtmlSanitizeEx.strip_tags/1 function on any change that satisfies all of the following conditions: The field associated with the change is of the type :string . The field associated with the change is not in the blacklisted_fields list of opts as defined using the :except key in opts . Note that this function will change the value in the :changes map of an %Ecto.Changeset{} struct if the given changes are sanitized. Examples iex&gt; attrs = %{ string_field : &quot;&lt;script&gt;Bad&lt;/script&gt;&quot; } iex&gt; result_changeset = ...&gt; attrs ...&gt; |&gt; FakeEctoSchema . changeset ( ) ...&gt; |&gt; EctoSparkles.SanitiseStrings . strip_all_tags ( ) iex&gt; result_changeset . changes %{ string_field : &quot;Bad&quot; } Fields can be exempted from sanitization via the :except option. iex&gt; attrs = %{ string_field : &quot;&lt;script&gt;Bad&lt;/script&gt;&quot; } iex&gt; result_changeset = ...&gt; attrs ...&gt; |&gt; FakeEctoSchema . changeset ( ) ...&gt; |&gt; EctoSparkles.SanitiseStrings . strip_all_tags ( except : [ :string_field ] ) iex&gt; result_changeset . changes %{ string_field : &quot;&lt;script&gt;Bad&lt;/script&gt;&quot; } You can also specify a specific scrubber (by passing a function as reference): ies&gt; attrs ...&gt; |&gt; FakeEctoSchema . changeset ( ) ...&gt; |&gt; EctoSparkles.SanitiseStrings . sanitise_strings ( scrubber : HtmlSanitizeEx.Scrubber . html5 / 1 )","ref":"EctoSparkles.SanitiseStrings.html#strip_all_tags/2"},{"type":"extras","title":"EctoSparkles","doc":"Some helpers to sparkle on top of Ecto EctoSparkles.proload/3 and EctoSparkles.join_preload/2 to join and preload associations with less verbosity EctoSparkles.reusable_join/5 to avoid duplicating joins EctoSparkles.Migrator to run migrations, rollbacks, etc in a release and EctoSparkles.AutoMigrator to automatically run them at startup. EctoSparkles.DataMigration : a behaviour implemented for data migrations (generally backfills). EctoSparkles.Log to log slow or possible N+1 queries with telemetry (showing stacktraces) EctoSparkles.Changesets.Errors to generate readable errors for changesets NOTE: you need to put something like config :ecto_sparkles, :otp_app, :your_otp_app_name in your app's config.","ref":"readme.html"},{"type":"extras","title":"EctoSparkles - proload documentation","doc":"A macro which tells Ecto to perform a join and preload of associations. By default, Ecto preloads associations using a separate query for each association, which can degrade performance. You can make it run faster by using a combination of join/preload, but that requires a bit of boilerplate (see examples below). Examples using standard Ecto query |&gt; join ( :left , [ o , activity : activity ] , assoc ( :object ) , as : :object ) |&gt; preload ( [ l , activity : activity , object : object ] , activity : { activity , [ object : object ] } ) Ecto requires calling three different functions for this operation: Query.join/4 , Query.assoc/3 and Query.preload/2 . Here's another example: Invoice |&gt; join ( :left , [ i ] , assoc ( i , :customer ) , as : :customer ) |&gt; join ( :left , [ i ] , assoc ( i , :lines ) , as : :lines ) |&gt; preload ( [ lines : v , customers : c ] , lines : v , customer : c )","ref":"readme.html#proload-documentation"},{"type":"extras","title":"EctoSparkles - Example using proload","doc":"With proload , you can accomplish this with just one line of code: proload ( query , activity : [ :object ] ) And for the other example: proload ( Invoice , [ :customer , :lines ] ) As a bonus, it automatically makes use of reusable_join so calling it multiple times with the same association has no ill effects.","ref":"readme.html#example-using-proload"},{"type":"extras","title":"EctoSparkles - Example using join_preload","doc":"join_preload is proload 's sister macro with a slightly different syntax: join_preload ( query , [ :activity , :object ] ) and: Invoice |&gt; join_preload ( :customer ) |&gt; join_preload ( :lines )","ref":"readme.html#example-using-join_preload"},{"type":"extras","title":"EctoSparkles - reusable_join documentation","doc":"A macro similar to Ecto.Query.join/{4,5} , but can be called multiple times with the same alias. Note that only the first join operation is performed, the subsequent ones that use the same alias are just ignored. Also note that because of this behaviour, its mandatory to specify an alias when using this function. This is helpful when you need to perform a join while building queries one filter at a time, because the same filter could be used multiple times or you could have multiple filters that require the same join, which poses a problem with how the filter/3 callback work, as you need to return a dynamic with the filtering, which means that the join must have an alias, and by default Ecto raises an error when you add multiple joins with the same alias. To solve this, it is recommended to use this macro instead of the default Ecto.Query.join/{4,5} , in which case there will be only one join in the query that can be reused by multiple filters. Creating reusable joins query |&gt; reusable_join ( :left , [ t1 ] , t2 in &quot;other_table&quot; , on : t1 . id == t2 . id , as : :other_a ) |&gt; reusable_join ( :left , [ t1 ] , t2 in &quot;other_table&quot; , on : t1 . id == t2 . id , as : :other_b )","ref":"readme.html#reusable_join-documentation"},{"type":"extras","title":"EctoSparkles - Copyright","doc":"Copyright (c) 2021 Bonfire developers Copyright (c) 2020 Up Learn Copyright (c) 2019 Joshua Nussbaum join_preload was originally forked from Ecto.Preloader , licensed under WTFPL) reusable_join was originally forked from QueryElf , licensed under Apache License Version 2.0 original code licensed under Apache License Version 2.0","ref":"readme.html#copyright"}]